/*******************************************************************************
* Copyright 2008 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/



using System;
using System.Collections.Generic;
using System.Text;

namespace AddDefect
{
    class Program
    {
        static void Main(string[] args)
        {
            //Define variables to hold the values passed by the arguments
            //TestTrack SOAP Serer URL
            string ttURL = args[0];
            //TestTrack project name
            string ttProject = args[1];
            //TestTrack user to log in via SOAP
            string ttUser = args[2];
            //TestTrack password for above user
            string ttPassword = args[3];
            //Comment field in CodeCollab Defect - mapped to the Description field in TTP
            string dDescription = args[4];
            //Severity field in CodeCollab Defect - mapped to the Severity field in TTP
            string dSeverity = args[5];
            //Type field in CodeCollab Defect - mapped to the Type field in TTP
            string dType = args[6];
            //Custom field called Summary in CodeCollab Defect - mapped to the Summary field in TTP
            string dSummary = args[7];
            //User that performed action in CodeCollab, expected in the FirstName LastName format - mapped to the Entered by field in TTP
            string dFoundBy = args[8];
            //CodeCollab Defect id
            string collabDID = args[9];
            //CodeCollab Review id
            string collabRID = args[10];
            //The CodeCollab review id and defect id are used to create something for reference
            string dReference = collabRID + ":" + collabDID;
         
            
            //Process the username to be LastName, FirstName
            string fName = dFoundBy.Substring(0, dFoundBy.IndexOf(" "));
            string lName = dFoundBy.Substring(dFoundBy.IndexOf(" ") + 1);
            dFoundBy = lName + ", " + fName;
            //Boolean used to help check if defect exists
            bool dExists = false;

            //Instantiate a soap object
            ttsoapcgi ttSoap = new ttsoapcgi();
            //Set the URL
            ttSoap.Url = ttURL;
            //Cookie used to tell the soapcgi which project you are logged in
            long ttCookie = 0;
            
            //Get list of projects and determine which one we want to log into
            CProject[] ttCProjects = ttSoap.getProjectList(ttUser,ttPassword );
            //Define the project object will be loggin in
            CProject ttCProject = new CProject();
            for (int i = 0; i < ttCProjects.Length; i++)
            {
                if (ttCProjects[i].database.name == ttProject)
                {
                    ttCProject = ttCProjects[i];
                }
            }
            if (ttCProject != null)
            {
                //If we did find a project, log in
                ttCookie = ttSoap.ProjectLogon(ttCProject, ttUser, ttPassword);

                //Process to see if defect already exists, we'll need to get the defect table, and display the reference field for each one
                //First define the column group
                CTableColumn[] dColumns = new CTableColumn[1];
                dColumns[0] = new CTableColumn();
                dColumns[0].name = "Reference";
                //Next, get the defect table
                CDatabaseTable[] ttTables = ttSoap.getTableList(ttCookie );
                CRecordListSoap ttDefects = ttSoap.getRecordListForTable(ttCookie, ttTables[0].name.ToString(), "", dColumns);
                //Now go through each defect and see if we have a match in the reference field.
                for (int i = 0; i < ttDefects.records.Length; i++)
                {
                    if (ttDefects.records[i].row[0].value != null)
                    {
                        if (ttDefects.records[i].row[0].value.Equals(dReference))
                        {
                            dExists = true;
                        }
                    }
                }
                //If it does not exist, then add it.
                if (dExists == false)
                {
                    try
                    {
                        //Create the defect object
                        CDefect dAdd = new CDefect();
                        dAdd.summary = dSummary;
                        dAdd.severity = dSeverity;
                        dAdd.type = dType;
                        dAdd.reference = dReference;

                        //Create the reportby record that includes the summary and the enteredby
                        CReportedByRecord[] CDRecords = new CReportedByRecord[1];
                        CDRecords[0] = new CReportedByRecord();
                        CDRecords[0].comments = dDescription + " " + dFoundBy;
                        CDRecords[0].foundby = dFoundBy;
                        dAdd.reportedbylist = CDRecords;
            
                        //Add the defect
                        ttSoap.addDefect(ttCookie, dAdd);
                        //Log off
                        ttSoap.DatabaseLogoff(ttCookie);

                    
                    }
                    catch (Exception fcc)
                    {
                       Logger.log(fcc.Message);
                       //Log off in case exception caused it not log us off
                       if (ttCookie > 0)
                       {
                           ttSoap.DatabaseLogoff(ttCookie);
                       }
                    }
                }

            }
        }
    }
}
